@@ -53,13 +53,18 @@ struct _GgvPostScriptViewPrivate {
 	GgvSidebar *sidebar;
 
 	BonoboPropertyBag     *property_bag;
-	BonoboPropertyControl *property_control;
 
 	BonoboUIComponent     *uic, *popup_uic;
 
 	gboolean               zoom_fit;
 
 	gboolean pane_auto_jump, page_flip;
+
+	BonoboControl *owner;
+
+	GdkCursor *pan_cursor, *win_cursor;
+
+	GList *dialogs;
 };
 
 struct _GgvPostScriptViewClassPrivate {
@@ -83,12 +88,6 @@ enum {
 	PROP_STATUS
 };
 
-enum {
-	PROP_CONTROL_TITLE,
-};
-
-static GdkCursor *pan_cursor = NULL;
-
 static BonoboObjectClass *ggv_postscript_view_parent_class;
 
 static const gchar *orientation_paths[] = {
@@ -123,18 +122,6 @@ static const gchar *size_paths[] = {
 	NULL
 };
 
-void
-get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
-{
-	const gchar *status;
-
-	g_assert(arg->_type == TC_CORBA_string);
-	
-	status = GTK_GS(ps_view->priv->gs)->gs_status;;
-
-	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
-}
-
 static void
 get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
@@ -153,6 +140,47 @@ get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 	*(CORBA_string *)arg->_value = CORBA_string_dup(title);
 }
 
+static void
+get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
+{
+	const gchar *status;
+
+	g_assert(arg->_type == TC_CORBA_string);
+	
+	status = GTK_GS(ps_view->priv->gs)->gs_status;;
+
+	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
+}
+
+static void
+remove_dialog_from_list(GtkWidget *w, gpointer user_data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(user_data);
+
+	ps_view->priv->dialogs = g_list_remove(ps_view->priv->dialogs, w);
+}
+
+static void
+add_dialog_to_view(GgvPostScriptView *ps_view, GtkWidget *dlg)
+{
+	ps_view->priv->dialogs = g_list_prepend(ps_view->priv->dialogs, dlg);
+	g_signal_connect(G_OBJECT(dlg), "destroy",
+					 G_CALLBACK(remove_dialog_from_list), ps_view);
+}
+
+static void
+ggv_postscript_view_destroy_dialogs(GgvPostScriptView *ps_view)
+{
+	GList *dialog_node = ps_view->priv->dialogs;
+	GtkWidget *dialog;
+
+	while(dialog_node) {
+		dialog = GTK_WIDGET(dialog_node->data);
+		dialog_node = dialog_node->next;
+		gtk_widget_destroy(dialog);
+	}
+}
+
 static void
 set_file_items_sensitivity(GgvPostScriptView *ps_view, gboolean sens)
 {
@@ -252,15 +280,17 @@ notify_page_change(GgvPostScriptView *ps_view)
 	BonoboArg *arg;
 
 	set_page_items_sensitivity(ps_view);
-	ggv_sidebar_page_changed(ps_view->priv->sidebar,
-							 gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)));
+	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) >= 0) {
+		ggv_sidebar_page_changed(ps_view->priv->sidebar,
+								 gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)));
 
-	arg = bonobo_arg_new(TC_CORBA_long);
-	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
-	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
-										 "Bonobo/Property:change:page",
-										 arg, NULL);
-	bonobo_arg_release(arg);
+		arg = bonobo_arg_new(TC_CORBA_long);
+		*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+		bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+											 "Bonobo/Property:change:page",
+											 arg, NULL);
+		bonobo_arg_release(arg);
+	}
 }
 
 static void
@@ -315,10 +345,12 @@ ggv_postscript_view_get_page_count(GgvDocument *doc)
 	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
 }
 
-static void
+static gboolean
 load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 {
-	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	gboolean rv;
+
+	rv = gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
 	notify_title_change(ps_view);
 	notify_page_count_change(ps_view);
 	notify_orientation_change(ps_view);
@@ -327,6 +359,7 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
 	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	notify_status_change(ps_view);
+	return rv;
 }
 
 static void
@@ -405,7 +438,10 @@ load_ps_from_stream (BonoboPersistStream *ps,
 	} while (len_read > 0);
 
 	fclose(tmpfile);
-	load_ps(ps_view, ps_view->priv->tmp_name);
+	if(!load_ps(ps_view, ps_view->priv->tmp_name)) {
+		CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+							ex_GNOME_GGV_DocumentError, NULL);
+	}
 	return;
 
  exit_clean:
@@ -434,13 +470,14 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 		gtk_gs_start_scroll(GTK_GS(ps_view->priv->gs));
 
 		ps_view->priv->pan = TRUE;
-		if(pan_cursor == NULL)
-			pan_cursor = cursor_get(widget->window, CURSOR_HAND_CLOSED);
+		if(ps_view->priv->pan_cursor == NULL)
+			ps_view->priv->pan_cursor =
+				cursor_get(widget->window, CURSOR_HAND_CLOSED);
 		gtk_grab_add(widget);
 		gdk_pointer_grab(widget->window, FALSE,
 						 GDK_POINTER_MOTION_MASK |
 						 GDK_BUTTON_RELEASE_MASK, NULL,
-						 pan_cursor, GDK_CURRENT_TIME);
+						 ps_view->priv->pan_cursor, GDK_CURRENT_TIME);
 		ps_view->priv->prev_x = wx;
 		ps_view->priv->prev_y = wy;
 
@@ -523,6 +560,7 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 	GgvPostScriptView *ps_view;
 	gchar *local_file = NULL, *uri_str;
 	FILE *tmpfile;
+	gint rv;
 
 	g_return_val_if_fail (data != NULL, -1);
 	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (data), -1);
@@ -573,15 +611,24 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 		if(result != GNOME_VFS_ERROR_EOF)
 			return -1;
 	}
+	rv = 0;
 	if(ps_view->priv->tmp_name) {
-		load_ps(ps_view, ps_view->priv->tmp_name);
+		if(!load_ps(ps_view, ps_view->priv->tmp_name)) {
+			CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+								ex_GNOME_GGV_DocumentError, NULL);
+			rv = -1;
+		}
 	}
 	else if(local_file) {
-		load_ps(ps_view, local_file);
+		if(!load_ps(ps_view, local_file)) {
+			CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+								ex_GNOME_GGV_DocumentError, NULL);
+			rv = -1;
+		}
 		g_free(local_file);
 	}
 
-	return 0;
+	return rv;
 }
 
 static void
@@ -637,7 +684,12 @@ sync_orientation_items(GgvPostScriptView *ps_view)
 {
 	gint i;
 	gboolean orient_state;
-	
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (ps_view->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
 	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
 	for(i = 0; orientation_paths[i] != NULL; i++) {
 		if(i > 0)
@@ -657,6 +709,11 @@ sync_size_items(GgvPostScriptView *ps_view)
 	gint i;
 	gboolean size_state;
 	const gchar **paths;
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (ps_view->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
 
 	paths = size_paths;
 
@@ -740,13 +797,16 @@ ggv_postscript_view_get_ps(GgvPostScriptView *ps_view, gint *active_rows)
 static void
 ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 {
-	gchar **argv;
-	gint print_in;
+	gchar **argv = NULL;
+	gint print_in = -1;
 
-	argv = g_strsplit(ggv_print_cmd, " ", 0);
-	if(!g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+	if((NULL == ggv_print_cmd) ||
+	   ((argv = g_strsplit(ggv_print_cmd, " ", 0)) == NULL) ||
+	   (argv[0] == NULL) ||
+	   (!g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 								 NULL, NULL, NULL, &print_in, NULL, NULL,
-								 NULL)) {
+								 NULL)) ||
+	   (print_in < 0)) {
 		GtkWidget *dlg;
 		dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
 									 GTK_DIALOG_MODAL,
@@ -754,6 +814,12 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 									 GTK_BUTTONS_OK,
 									 _("Unable to execute print command:\n%s"),
 									 ggv_print_cmd);
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}
 		gtk_widget_show(dlg);
 		gtk_dialog_run(GTK_DIALOG(dlg));
 		gtk_widget_destroy(dlg);
@@ -762,7 +828,8 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 		write(print_in, ps, strlen(ps));
 		close(print_in);
 	}
-	g_strfreev(argv);
+	if(NULL != argv)
+		g_strfreev(argv);
 }
 
 static gboolean
@@ -802,12 +869,18 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	if(active_rows[0] == -1) {
 		GtkWidget *dlg;
 		
-		dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
-									 GTK_DIALOG_MODAL,
+		dlg = gtk_message_dialog_new(NULL,
+									 0,
 									 GTK_MESSAGE_QUESTION,
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to save the whole document?"));
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}
 		gtk_widget_show(dlg);
 		switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 		case GTK_RESPONSE_NO:
@@ -829,19 +902,30 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 		GtkWidget *chooser;
 
 		chooser = gtk_file_chooser_dialog_new(_("Select a file to save pages as"),
-						      GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
-						      GTK_FILE_CHOOSER_ACTION_SAVE,
-						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						      GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-						      NULL);
+											  GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+											  GTK_FILE_CHOOSER_ACTION_SAVE,
+											  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+											  GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+											  NULL);
+		add_dialog_to_view(ps_view, GTK_WIDGET(chooser));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(chooser), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(chooser), TRUE);
+		}
 		gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
 		gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
 		if (ps_view->priv->save_path)
 			gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(chooser),
-						 		ps_view->priv->save_path);
-
-		if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
-			fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+													ps_view->priv->save_path);
+		
+		do {
+			fname = NULL;
+			if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+				fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+			else
+				break;
+		} while(NULL == fname);
 
 		gtk_widget_destroy(chooser);
 
@@ -857,6 +941,12 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 											 GTK_BUTTONS_YES_NO,
 											 _("A file with this name already exists.\n"
 											   "Do you want to overwrite it?"));
+				add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
+				if(ps_view->priv->owner != NULL) {
+					bonobo_control_set_transient_for(ps_view->priv->owner,
+													 GTK_WINDOW(dlg), NULL);
+					gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+				}
 				gtk_widget_show(dlg);
 				switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 				case GTK_RESPONSE_NO:
@@ -896,6 +986,12 @@ verb_FilePrintMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to print the whole document?"));
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}
 		gtk_widget_show(dlg);
 		switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 		case GTK_RESPONSE_NO:
@@ -1091,6 +1187,8 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 		g_assert(arg->_type == TC_CORBA_long);
 
 		*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+		if((*(CORBA_long *)arg->_value) < 0)
+			*(CORBA_long *)arg->_value = 0;
 		break;
 	}
 	case PROP_PAGE_COUNT: {
@@ -1381,8 +1479,10 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 			continue;
 		}
 
-		if (!strcmp (name, "control"))
+		if (!strcmp (name, "control")) {
 			object = (BonoboObject *) ggv_control_new (ps_view);
+			ps_view->priv->owner = (BonoboControl *)object;
+		}
 #ifdef ENABLE_GGV_EMBEDDABLE
 		else if (!strcmp (item_name, "embeddable"))
 			object = (BonoboObject *) ggv_embeddable_new (image);
@@ -1466,17 +1566,6 @@ ggv_postscript_view_get_property_bag(GgvPostScriptView *ps_view)
 	return ps_view->priv->property_bag;
 }
 
-BonoboPropertyControl *
-ggv_postscript_view_get_property_control(GgvPostScriptView *ps_view)
-{
-	g_return_val_if_fail(ps_view != NULL, NULL);
-	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), NULL);
-
-	bonobo_object_ref(BONOBO_OBJECT(ps_view->priv->property_control));
-
-	return ps_view->priv->property_control;
-}
-
 void
 ggv_postscript_view_set_ui_container(GgvPostScriptView *ps_view,
 									 Bonobo_UIContainer ui_container)
@@ -1625,8 +1714,18 @@ ggv_postscript_view_destroy(BonoboObject *object)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->sidebar));
 		ps_view->priv->sidebar = NULL;
 	}
+	if(ps_view->priv->pan_cursor) {
+		gdk_cursor_unref(ps_view->priv->pan_cursor);
+		ps_view->priv->pan_cursor = NULL;
+	}
+	if(ps_view->priv->win_cursor) {
+		gdk_cursor_unref(ps_view->priv->win_cursor);
+		ps_view->priv->win_cursor = NULL;
+	}
 	ps_view_clean_tmp_file(ps_view);
 
+	ggv_postscript_view_destroy_dialogs(ps_view);
+
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
 }
@@ -1794,61 +1893,6 @@ ggv_postscript_view_get_type (void)
     return type;
 }
 
-#if 0
-static void
-property_control_get_prop(BonoboPropertyBag *bag,
-						  BonoboArg         *arg,
-						  guint              arg_id,
-						  CORBA_Environment *ev,
-						  gpointer           user_data)
-{
-	switch(arg_id) {
-	case PROP_CONTROL_TITLE:
-		g_assert(arg->_type == BONOBO_ARG_STRING);
-		BONOBO_ARG_SET_STRING(arg, _("Display"));
-		break;
-	default:
-		g_assert_not_reached();
-	}
-}
-#endif
-
-static BonoboControl *
-property_control_get_cb(BonoboPropertyControl *property_control,
-						int page_number, void *closure)
-{
-	GgvPostScriptView *ps_view;
-
-	g_return_val_if_fail(closure != NULL, NULL);
-	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(closure), NULL);
-	g_return_val_if_fail(page_number == 0, NULL);
-
-	ps_view = GGV_POSTSCRIPT_VIEW(closure);
-
-#if 0
-	container = eog_create_preferences_page(ps_view, page_number);
-
-	gtk_widget_show_all(container);
-
-	control = bonobo_control_new(container);
-
-	/* Property Bag */
-	property_bag = bonobo_property_bag_new(property_control_get_prop,
-										   NULL, control);
-
-	bonobo_property_bag_add(property_bag, "bonobo:title",
-							PROP_CONTROL_TITLE, BONOBO_ARG_STRING,
-							NULL, NULL, BONOBO_PROPERTY_READABLE);
-
-	bonobo_object_add_interface(BONOBO_OBJECT(control),
-								BONOBO_OBJECT(property_bag));
-
-	return control;
-#else
-	return NULL;
-#endif
-}
-
 static void
 interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 {
@@ -1878,11 +1922,12 @@ interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
 static void
 view_realized_cb(GtkGS *gs, gpointer data)
 {
-	GdkCursor *cursor;
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	cursor = cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
-	gdk_window_set_cursor(GTK_WIDGET(gs)->window, cursor);
-	gdk_cursor_unref(cursor);
+	if(ps_view->priv->win_cursor == NULL)
+		ps_view->priv->win_cursor =
+			cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
+	gdk_window_set_cursor(GTK_WIDGET(gs)->window, ps_view->priv->win_cursor);
 }
 
 static void
@@ -2012,7 +2057,7 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 	g_signal_connect(G_OBJECT(gs), "interpreter_error",
 					 G_CALLBACK(interpreter_error_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "realize",
-					 G_CALLBACK(view_realized_cb), NULL);
+					 G_CALLBACK(view_realized_cb), ps_view);
 
 	gtk_widget_show(GTK_WIDGET(gs));
 
@@ -2081,10 +2126,6 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							_("Antialiasing"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
 
-	/* Property Control */
-	ps_view->priv->property_control = bonobo_property_control_new
-		(property_control_get_cb, 1, ps_view);
-
 	/* UI Component */
 	ps_view->priv->uic = bonobo_ui_component_new("GgvPostScriptView");
 
