@@ -316,7 +316,7 @@ gtk_check_list_get_active_list(GtkCheckList *check_list)
         gint *retval;
         gboolean val;
 
-        if(!gtk_tree_model_get_iter_root(GTK_TREE_MODEL(check_list->list_model),
+        if(!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(check_list->list_model),
                                          &iter)) {
                 retval = g_new(gint, 1);
                 retval[0] = -1;
@@ -333,8 +333,8 @@ gtk_check_list_get_active_list(GtkCheckList *check_list)
                                          &iter));
 
         retval = g_new(gint, n + 1);
-        gtk_tree_model_get_iter_root(GTK_TREE_MODEL(check_list->list_model),
-                                     &iter);
+        gtk_tree_model_get_iter_first(GTK_TREE_MODEL(check_list->list_model),
+                                      &iter);
         i = j = 0;
         while(i < n) {
                 gtk_tree_model_get(GTK_TREE_MODEL(check_list->list_model),
